#!/bin/sh
#
# This script will set the password for the specified username to
# the provided value and set the account so that the user will have
# to change the password on the next login.  
#
# This script is intended as an example of using shell scripts with
# Sudo policy to delegate privileged operations to users, who have a 
# need to perform these operations as part of their job description,
# without having to give these users the root password.
#
# Should be run on a machine which has TAMOS configured and is
# subscribed to a branch with the sample sudo policy defined.
#  Usage:
#      $ pdossudo set_passwd  <username> <password>
#
# Examples uses for this script:  TBD
#   1. Used as a pre-config check of the certificate file to check
#      its lifetime and to make sure yo can talk to the  User Registry server 
#      over SSL using this certificate.
#   2. Used to check the lifetime of the certificate to determine if
#      if it time to refresh the certificate. Note: The refresh of the
#      certificate used by AMOS to communicate with User Registry server is a
#      manual process. An administrator must check the lifetime of the ldapcert
#      as part of routine maintenance.
#
# Define many things
PASSWDCMD="/usr/bin/passwd"

### M A I N ###
#
# Check that we have an argument:
if [ $# -eq 1 ]; then
    USERNAME=$1
else
    echo "Usage: set_passwd.sh <username>"
    exit 1;
fi

#
# Check if we are asking for help  -?
if [ "$1" = "-?" ]; then
    echo "Usage: set_passwd.sh <username>"
    exit 1;
fi

#
# Run the passwd command to set the user's password
if [ -f $PASSWDCMD ]; then
	# Set the specified user's password to the provided value
	${PASSWDCMD} ${USERNAME};
	case $? in
   	0) # Run the passwd -f command to set that the user must change the password on next login
	   ${PASSWDCMD} -f ${USERNAME}
	   case $? in
   	   0) exit 0;
      	   ;;
   	   *) exit 1;
      	   ;;
	   esac
      	;;
   	*) echo "Running the passwd command failed"
	   exit 1;
      	;;
	esac
else
	echo "Could not find the passwd command"
	exit 1;
fi

exit 0
