#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

# common code for ACL conversion

from com.ibm.amos.agent.db import PolicyConstants
from com.ibm.amos.agent.db.rules import ACL
from com.ibm.amos.nls.pdopmidx import *
from com.ibm.amos.agent.db.PolicyConstants import RESOURCE_OWNER, RESOURCE_OWNING_GROUP

def copyAttrIfSet(src, dest, attr):
    v = src.getExtAttributeValue(attr, 0)
    if v is not None:
        dest.addExtAttribute(attr, v)



def convertFromTAMOS(source, result, currentFile, currentAcl, policyFile, policyAcl, mapACLBits, bitmap):
    """
    Generic code for TAMOS->native ACL conversion.
    Takes care of mapping all permission bits, transferring ownership attributes,
    warning for user/group entries matching the file owner.  Obviously, this only
    works for ACL formats that include the concept of 'owner' and 'owning group'
    entry types.
    """
    resultAcl = ACL()
    resultAcl.setSource(source)
    resultAcl.setKey(policyAcl.getKey())

    # convert basic entries
    ownerPerms = mapACLBits(bitmap, policyAcl.getOwnerPermission())
    if ownerPerms is None:
        ownerPerms = currentAcl.getOwnerPermission()
    resultAcl.setOwnerPermission(ownerPerms)

    owningGroupPerms = mapACLBits(bitmap, policyAcl.getOwningGroupPermission())
    if owningGroupPerms is None:
        owningGroupPerms = currentAcl.getOwningGroupPermission()
    resultAcl.setOwningGroupPermission(owningGroupPerms)

    anyOtherPerms = mapACLBits(bitmap, policyAcl.getAnyOtherPermission())
    if anyOtherPerms is None:
        anyOtherPerms = currentAcl.getAnyOtherPermission()
    resultAcl.setAnyOtherPermission(anyOtherPerms)

    # figure out who will own the file once we've applied the policy, so we can identify
    # user and group entries that apply to the owner and owning group
    fileOwner = policyFile.getExtAttributeValue(RESOURCE_OWNER, 0) or \
                policyAcl.getExtAttributeValue(RESOURCE_OWNER, 0) or \
                currentFile.getExtAttributeValue(RESOURCE_OWNER, 0)
    fileOwningGroup = policyFile.getExtAttributeValue(RESOURCE_OWNING_GROUP, 0) or \
                policyAcl.getExtAttributeValue(RESOURCE_OWNING_GROUP, 0) or \
                currentFile.getExtAttributeValue(RESOURCE_OWNING_GROUP, 0)

    # copy ownership attributes to result ACL
    copyAttrIfSet(policyAcl, resultAcl, RESOURCE_OWNER)
    copyAttrIfSet(policyAcl, resultAcl, RESOURCE_OWNING_GROUP)

    # convert user entries
    for user in policyAcl.userMap.keySet():
        # special case: drop entries for sec_master that don't grant any relevant permissions
        if user == 'sec_master' and policyAcl.getUserPermission(user) == 'T':
            continue

        mapped = mapACLBits(bitmap, policyAcl.getUserPermission(user))
        if user == fileOwner:
            # warn about acl entries that don't achieve anything (but still apply them)
            result.actionWarning(npm_m_fsacl_file_owner_entry_found, [policyAcl.getKey(), user, policyFile.getKey()])

        resultAcl.addUserPermission(user, mapped)

    # convert group entries
    for group in policyAcl.groupMap.keySet():
        mapped = mapACLBits(bitmap, policyAcl.getGroupPermission(group))
        if group == fileOwningGroup:
            # warn about acl entries that don't achieve anything (but still apply them)
            result.actionWarning(npm_m_fsacl_file_owning_group_entry_found, [policyAcl.getKey(), group, policyFile.getKey()])

        resultAcl.addGroupPermission(group, mapped)

    return resultAcl

# vim: set ts=4 sw=4 expandtab :
