#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

from com.ibm.amos.agent.npm import PluginWrappedException

import sys
import traceback

#
# module for code shared between plugins
#

def logPyException(logger, exception):
	etype, value, tb = sys.exc_info()
	fmt = traceback.format_exception(etype, value, tb)

	# could probably get source object/method here, but
	# I'm not quite interested enough to get that right
	logger.severe("%s" % "".join(fmt))

def wrapPyException(plugin, exception):
	"""
	Returns a Java exception object wrapping the provided python exception.
	"""
	etype, value, tb = sys.exc_info()

	# rearrange the exception message and traceback to be a bit more javalike
	emsg = traceback.format_exception_only(etype, value)
	tbmsg = traceback.format_tb(tb)

	msg = "".join(emsg) + "".join(tbmsg)
	return PluginWrappedException(plugin, msg)


def escapeSplit(line, delim):
    """
    Splits a string using a delimiter, ignoring cases where the delimiter character
    is escaped using a backslash.  Unescapes the delimiter, but no other characters.
    """
    bits = []
    sl = False
    bit = ""
    for c in line:
        if c == '\\':
            sl = not sl
        elif c == delim:
            if sl == False:
                bits.append(bit)
                bit = ""
            else:
                bit += c
                sl = False
        else:
            if sl:
                bit += "\\"
                sl = False
            bit += c

    if sl:
        bit += "\\"
    bits.append(bit)

    return bits


def unescape(str):
    """
    Removes backslash escaping from a string, effectively replacing '\q' with 'q'.
    """
    r = ""
    sl = False
    for c in str:
        if sl:
            r += c
            sl = False
        elif c == '\\':
            sl = True
        else:
            r += c

    return r



def escapeJoin(bits, delim, esc=""):
    """
    Constructs a string by joining a sequence of strings with a delimiter, escaping
    the delimiter with a backslash anywhere it appears in the strings being joined.
    """
    r = ""
    for bit in bits:
        if r != "":
            r += delim

        for c in bit:
            if c == delim or c in esc:
                r += '\\'
            r += c

    return r

def splitDirName(filename):
    """
    Splits a file path into directory and basename components.
    """
    lastslash = filename.rfind('/')
    if lastslash == -1:
        dir = "/"
        filename = filename
    else:
        dir = filename[0:lastslash] or "/"
        filename = filename[lastslash+1:]

    return (dir, filename)

# vim: set ts=4 sw=4 expandtab :
