#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

# This class is an implementation of the StanzaFile abstract class for use with
# the AIX configuration files

from ossutil.configeditor import StanzaFile

class AixSystemConfigFile(StanzaFile):
    def __init__(self, data):
        StanzaFile.__init__(self, data, False, False, "")

    def determineStanzaName(self, line):
        """
        Determines whether or not the line is a stanza start line. In the AIX
        auditing configuration files, Stanza start lines are in the form
        <stanza name>:
        """
        if line:
            line = line.rstrip()
            if len(line) > 0 and  line[0] != ' ' and line[0] != '#' and line[0] != '\t' and line[-1] == ':':
                return line[:-1]

        return None

    def determineStanzaLine(self, name):
        """
        Get the line that would be used to identify a stanza of the name
        specified. For AIX auditing configuration files, this is in the form
        <stanza name>:
        """
        if name:
            return name + ":"
        return None

    def convertToLine(self, name, values):
        """
        Convert a name and a list of values to a configuration entry. This is
        a line in the form
            name = value[,value]*
        """
        if name and values:
            return "\t%s = %s" % (name, ",".join(values))
        return None

    def appendEntry(self, stzName, name, values):
        """
        Add an entry to the end of a stanza.
        """
        StanzaFile.append(self, stzName, self.convertToLine(name, values))

    def updateStz(self, stanza, callback, cbdata):
        def cb(stanzaName, line, data):
            rv = line

            # Always keep the stanza name unmodified
            if self.determineStanzaName(line) is not None:
                return line

            try:
                (name, valuestr) = line.split("=")
                values = valuestr.split(",")
                stripname = name.strip()
                def stripcb(item):
                    return item.strip()
                stripvalues = map(stripcb, values)

                newvalues = callback(stanzaName, stripname, stripvalues, data)

                if newvalues is not None:
                    if newvalues == stripvalues:
                        rv = line
                    else:
                        rv = self.convertToLine(stripname, newvalues)
                else:
                    rv = None

            except ValueError:
                # If the split fails, then it means this isn't a valid entry
                # in the stanza, so kick it out
                rv = None

            return rv

        stanza.update(cb, cbdata)
        if stanza.hasChanged():
            self.changed = True

