#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

#
# key-value (stanzaless) config file editor
#

from configeditor import LineEditor
import re

class KeyValueEditor:
    """
    Simple editor for configuration files based on 'key = value' lines.
    """
    def __init__(self, data, has_comments=True, has_continuations=False):
        self.edit = LineEditor(data, has_comments=False, has_continuations=has_continuations)
        self.has_comments = has_comments

    def _update(self, key, replacement, append):
        done = [False]
        if self.has_comments:
            mush = re.compile("^(\s*#)?\s*" + key + "\s*=")
        else:
            mush = re.compile("^\s*" + key + "\s*=")

        def cb(line, done):
            if mush.match(line):
                if done[0]:
                    return None

                done[0] = True
                if replacement == '###':
                    return '# ' + line
                else:
                    return replacement
            else:
                return line

        self.edit.update(cb, done)
        if append and not done[0]:
            self.edit.append(replacement)

    def set(self, key, value):
        if value is None:
            self.unset(key)
        else:
            self._update(key, "%s=%s" % (key, value), True)

    def unset(self, key):
        if self.has_comments:
            self._update(key, "###", False)
        else:
            self._update(key, None, False)

    def get(self, key):
        mush = re.compile("^\s*" + key + "\s*=")
        v = None
        for l in self.edit.lines():
            if mush.match(l):
                v = l[l.find('=')+1:].strip()
        return v


    def hasChanged(self):
        return self.edit.hasChanged()

    def getData(self):
        return self.edit.getData()

# vim: set ts=4 sw=4 expandtab :
