#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

#
# utilities for policy difference reporting
#

def getMapKeys(map):
    if hasattr(map, "keys"):
        return map.keys()
    elif hasattr(map, "keySet"):
        return map.keySet()
    else:
        # now what?
        return []

def compareStringMap(result, itemtype, before, after):
    """
    Compares before and after string maps, reporting all differences
    through the PluginResult object given, and returning True iff the
    maps are equivalent.
    """
    r = True
    # might be worth sorting key lists here, for consistency.
    for k in getMapKeys(before):
        if k not in after:
            r = False
            result.diff(itemtype, k, before[k], None)
        elif before[k] != after[k]:
            r = False
            result.diff(itemtype, k, before[k], after[k])

    for k in getMapKeys(after):
        if k not in before:
            r = False
            result.diff(itemtype, k, None, after[k])

    return r

def compareList(result, itemtype, before, after):
    """
    Compares two (string) lists, reporting entries added and removed,
    returning True iff the lists are equivalent.  Order is significant
    here; use compareSet() for order-insensitive comparison.
    """
    r = True

    # pretty naive
    for i in before:
        if i not in after:
            result.diff(itemtype, i, 1, 0)
            r = False

    for i in after:
        if i not in before:
            result.diff(itemtype, i, 0, 1)
            r = False

    return r

def compareSet(result, itemtype, before, after):
    """
    Compares two (string) sets, reporting entries added and removed,
    returning True iff the sets are equivalent.  Ordering is ignored
    here; [a, b] is considered equal to [b, a]
    """
    sb = before[:]
    sb.sort()
    sa = after[:]
    sa.sort()
    return compareList(result, itemtype, sb,sa)



# vim: set ts=4 sw=4 expandtab :
