#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

#
# general Solaris utilities
#

from __future__ import generators

from ossutil.configeditor import LineEditor
import java.lang.String

class SolarisShadowFile:
    """
    Specialised editor for the Solaris /etc/shadow file.
    Since this is something of a crucial file, the update method is
    a bit restricted (it won't remove lines), and there is no append method.
    We should never create or remove users in this fashion.
    """

    def __init__(self, data):
        # correct?
        self.edit = LineEditor(java.lang.String(data), has_comments=False, has_continuations=False)
        # we present records to callers as dicts containing these fields
        self.fields = ("username", "password", "lastchg", "min", "max", "warn", "inactive", "expire", "flag")


    def __split(self, line):
        f = line.split(':')
        if len(f) != len(self.fields):
            # broken line, skip it
            return None

        return dict(zip(self.fields, f))

    def entries(self):
        for line in self.edit.lines():
            r = self.__split(line)
            if r is not None:
                yield r

    def update(self, callback, data):
        def cb(line, (callback, data)):
            record = self.__split(line)
            if record is None:
                return line

            updated = callback(record, data)
            if updated is None:
                return line 
 
            # require a username
            if updated.get('username') in (None, ""):
                return line

            # will probably throw an exception if 'updated' is malformed
            return ":".join([ updated[x] or "" for x in self.fields ])

        self.edit.update(cb, (callback, data))

    def getData(self):
        return self.edit.data

    def hasChanged(self):
        return self.edit.hasChanged()

