#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

#
# user registry utilities
#

import java.lang.StringBuffer

import platform

def getentExpandGroup(target, group):
    """
    Use getent to perform group expansion.
    """
    out = java.lang.StringBuffer()
    err = java.lang.StringBuffer()
    rc = target.execute(("/usr/bin/getent", "group", group), None, out, err)
    if rc != 0:
        # huh
        # probably log stderr?
        # raise exception?
        return []

    data = str(out.toString()).rstrip("\n")

    # get members
    ls = data.rfind(":")
    if ls == -1:
        # what?
        # log something, raise exception..
        return []

    mlist = data[ls+1:]
    if mlist == "":
        return []
    else:
        return mlist.split(",")

def lsgroupExpandGroup(target, group):
    """
    Use lsgroup to perform group expansion.
    """
    out = java.lang.StringBuffer()
    err = java.lang.StringBuffer()
    rc = target.execute(("/usr/sbin/lsgroup", "-a", "users", group), None, out, err)
    if rc != 0:
        # huh
        # probably log stderr?
        # raise exception?
        return []

    data = str(out.toString().rstrip("\n"))

    expectStr = group + " users="
    if data.find(expectStr) == 0:
        # Good
        userList = data[len(expectStr):]
        if len(userList) > 0:
            return userList.split(",")
        else:
            return []
    else:
        # Bad
        return []
    

def expandGroup(target, group):
    """
    Expands group membership, returning a list of the names of all users with
    the specified group as a supplemental group membership.

    This does not return users with the specified group as their primary group.
    It seems that we cannot in general identify group members via primary group
    membership.
    """
    plat = platform.getPlatformType()

    if plat == platform.PLATFORM_AIX:
        return lsgroupExpandGroup(target, group)
    else:
        return getentExpandGroup(target, group)
    


# vim: set ts=4 sw=4 expandtab :
