#
# IBM Confidential
# OCO Source Materials
# Tivoli Access Manager for Operating Systems - Agent
# TIVO-TAMOSA
#
# (C) Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

from com.ibm.amos.agent.db import PolicyConstants
from com.ibm.amos.agent.db.resources import User
from com.ibm.amos.nls.pdopmidx import *

from ossutil.userregistry import expandGroup

def auditPolicyExists(policy):
    """
    Returns True if any user audit policy exists
    """
    
    if policy is None:
        return False

    users = policy.resourceMap[PolicyConstants.USER]
    if users is not None:
        for user in users:
            if users[user].getAuditID() is not None:
                return True

    groups = policy.resourceMap[PolicyConstants.GROUP]
    if groups is not None:
        for group in groups:
            if groups[group].getAuditID() is not None:
                return True

    return False


def determineAuditUsers(target, policy):
        """
        Figures out which audit settings must be applied to which users.
        Returns a map of usernames to lists of audit setting objects that
        apply to that user, where the first item in the list is the user
        audit setting (or None if there isn't one), and the following items
        are any group audit settings that apply.

        Also returns an indication of whether any trace_exec audit settings
        were found in the policy.
        """
        users = {}
        haveExec = [False]

        def processSubject(users, policy, target, resource, expand, apply):
            if resource.getAuditID() is None:
                return

            audit = policy.getRule(PolicyConstants.AUDIT, resource.getAuditID())
            if audit is None:
                # complain
                return

            subjects = expand(target, resource)
            for s in subjects:
                if s not in users:
                    users[s] = [None]

                apply(users[s], audit)

            if 'exec' in audit.getLevels() or 'exec_l' in audit.getLevels():
                haveExec[0] = True

        target.logger.fine("Determining audit policy")

        userRes = policy.resourceMap[PolicyConstants.USER]
        if userRes is not None:
            for name in userRes:
                def userExpand(target, resource):
                    return (resource.getKey(),)

                def userApply(user, audit):
                    user[0] = audit

                processSubject(users, policy, target, userRes[name], userExpand, userApply)

        groupRes = policy.resourceMap[PolicyConstants.GROUP]
        if groupRes is not None:
            for name in groupRes:
                def groupExpand(target, resource):
                    members = expandGroup(target, resource.getKey())
                    target.getLogger().finer("Members of group %s: [%s]" % (resource.getKey(), ",".join(members)))
                    return members

                def groupApply(user, audit):
                    user.append(audit)

                processSubject(users, policy, target, groupRes[name], groupExpand, groupApply)

        target.logger.finest("Audit settings apply for users [%s]" % (",".join(users.keys())))
        return (users, haveExec[0])

# vim: set ts=4 sw=4 expandtab :
