#! /bin/sh
#
#  @(#)49  1.4 src/oss/common/pd_utils/ossdump.sh, oss.agent, ossagent600, 080929a 08/09/23 00:50:42 @(#)
#
#
#
# 5698-PDO
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2002,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
#set -x

####################################################################
####################################################################
#
# Data collection script to extract useful data for diagnosing
# potential AMOS problems.
#
####################################################################
####################################################################

# This script is intended for use by IBM Customer Support. To enforce
# data collection in English, set LANG and LC_ALL to C.
LANG=C
LC_ALL=C

SUM=/usr/bin/sum
LS=/bin/ls

TAILSIZE="200"
PROGRAM=ossdump.sh


##########################################################
##########################################################
# Functions common to all platforms
##########################################################
##########################################################


major_section() {
    stitle=$1
    print ""
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print "OOO  $stitle"
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print ""
}

#
# Usage message
#
usage() {
  print "usage: $PROGRAM [-g] [-l] [-s] [-V] [-R <release/fix level>] [-?|h]"
  print "  A minimum of one argument must be specified."
  print "  -g    General Information"
  print "  -l    Logfile Data"
  print "  -s    System Information"
  print "  -V    Version Information"
  print "  -R    Verify Installed Release Level"
  print "  -h|?  Display this Message"

  exit 1
}
#
# Handle optional arguments
#
ProcessArgs () {

  if [ $# -eq 0 ] ; then
    usage
  fi

  while getopts "glskKVh?R:" opt; do
    case $opt in
      g)
        General=yes;;
      l)  
        Logfile=yes;;
      s)  
        System=yes;;
      V)
	Version=yes;;
      R)
	VerifyRelease=yes
	ReleaseNumber=$OPTARG;;
      h)
		 usage;;
      \?)
		 usage;;
    esac
  done

  TMP=`expr $OPTIND - 1`
  shift $TMP

  if [ $# -ne 0 ] ; then
    print "bad arg count $#"
    usage
  fi
}

#
# This function wil set info on what the platform type is.
# We need this for plaform specific code.
#
SetMachineType() {
   PLATFORM=`/bin/uname`
   LEVEL=`/bin/uname -r`
       
   case $PLATFORM in
       AIX)   DF="/bin/df -k" 
              PS="/usr/bin/ps -elkf"
              MOUNT="/usr/sbin/mount"
		       INIT="/usr/sbin/lsitab -a"
              EFFID=`/usr/bin/id -u`
              TAIL="/usr/bin/tail -n "
              HEAD="/usr/bin/head -n "
              CAT=/usr/bin/cat
              WHAT=/usr/bin/what
	      print_to_stderr() {
		 print -u2 $1
	      }


#       case $LEVEL in
#           1) 
#              ;;
#           3) 
#              ;;
#           *) print "Unknown AIX level"
#       esac
       ;;
       SunOS) DF="/bin/df -k" 
              PS="/usr/bin/ps -elf"
              MOUNT="/usr/sbin/mount"
              EFFID=`/usr/xpg4/bin/id -u`
              TAIL="/usr/bin/tail -"
              HEAD="/usr/bin/head -"
              CAT=/usr/bin/cat
              WHAT=/usr/ccs/bin/what
              print () {
                echo $1
              }
	      print_to_stderr() {
                 echo $1 > /dev/stderr
	      }
       ;;
       HP-UX) DF="/bin/df -P"
              PS="/usr/bin/ps -elf"
              MOUNT="/usr/sbin/mount"
              PLATFORM="HPUX"
              EFFID=`/usr/bin/id -u`
              TAIL="/usr/bin/tail -n "
              HEAD="/usr/bin/head -n "
              CAT=/usr/bin/cat
              WHAT=/usr/bin/what
	      print_to_stderr() {
		 print -u2 $1
	      }
       ;;
       Linux) DF="/bin/df -k"
              PS="/bin/ps -elf"
              MOUNT="/bin/mount"
              EFFID=`/usr/bin/id -u`
              TAIL="/usr/bin/tail -n "
              HEAD="/usr/bin/head -n "
              CAT=/bin/cat
              print () {
                echo -e $1
              }
	      print_to_stderr() {
                 echo -e $1 > /dev/stderr
	      }
              
       ;;
       *)  print "Unknown Platform"
   esac
} 

####################################################################
#
# This unit describes what is collected for header information
#
####################################################################
buildHeader() {
    print_to_stderr ".... Building Header Information ...."
		  major_section "Header Information"

		  print "Hostname = `/bin/hostname`"
    print "Date     = `/bin/date`" 
    #print "Locale   = `/usr/bin/locale`"
    
    if [ $EFFID != 0 ] ; then
       print "Warning: collection of AMOS information should be done as root"
       print_to_stderr "Warning: collection of AMOS information should be done as root"
    fi
}

####################################################################
#
# This unit describes what is collected for header information
#
####################################################################


verifyReleaseLevel () {
    print_to_stderr ".... Verifying Release level of AMOS installed on the box...."
    major_section "Verify Release Level"

#	Get version number of AMOS as reported by pdosversion.
#	This gives you a baseline for the version of AMOS installed on the box.
#	If the version number reported by pdosversion does not match the user
#	supplied release level, bail out.
#	If the version match, then do a finer level verification.
#	Check the install database maintained by the Operating System to
#	find out of any of the AMOS files were changed after installation.
#	If so, report that these files did not pass the verificaiton check.
#	Note: This mechanism only works for release/patch level which are installed
#	via the "patch install" mechanism and will not work with fixes delivered
#	as a tar file. 

	UserSuppliedReleaseNumber=$1
	AMOSReleaseNumber=`pdosversion | grep pdosversion | awk '{print $2}'`
        if [ $UserSuppliedReleaseNumber = $AMOSReleaseNumber ]; then
             print "Expected AMOS release number ($UserSuppliedReleaseNumber) MATCHES installed version number"
             print "Below is a list of the files which have been changed since the product was installed."

    	case $PLATFORM in
       	     AIX) 
                 lppchk -c PDOS.agent
                 ;;
       	     SunOS) 
                 # The output for each file comes on multiple lines,
                 # so instead of suppressing the error of /var/pdos/audit
                 # just print a message asking users to ignore certain files.
                 print "Ignore version check failures in the /var/pdos/audit directory" 
                 pkgchk PDOSagent
                 ;;
       	     HPUX) 
             # This involves some work as there is no system command
             # to verify version. We need to rely on the output of
             # swlist -l file to give us the expected value of the
             # file cksum and then compare these values to the actual
             # cksum of the files.
             # Get a list of the files in /opt/pdos along with their cksum and size
             # and save it in a temporary file
             swlist -l file -a cksum -a size PDOSagent | \
             awk ' NF == 3 && /\/opt\/pdos/ { print $2 " " $3 " " $1 } ' \
              > /tmp/.pdos_match_failed
             
             # Extract the file names from the temp file
             x=`awk '{ print $3 }' /tmp/.pdos_match_failed`
             
             # For each file, calculate the actual cksum and compare
             # it to the expected cksum which is the value returned by swlist.
             # If the actual and expected values do not match, print an error.
             for i in $x
             
             do
             y=`cksum $i`
             z=`grep "$i$" /tmp/.pdos_match_failed`
             
             if [ "$y" != "$z" ]; then
                print "check failed: (actual cksum --- $y)  (expected cksum --- $z)"
             fi
             done
             
             print "Done with version check"
             # Remove temp file
             rm -rf /tmp/.pdos_match_failed

                 ;;
       	     Linux) 
#                 Construct Package Name
                  pkg=`rpm -qa | grep PDOS`
#                 Ignore changes to /var/pdos/audit 
         	  rpm -V $pkg | grep -v /var/pdos/audit
                 ;;
                 *)  print "Unknown Platform"
                 ;;
        esac

        else
             print "Expected AMOS release number ($UserSuppliedReleaseNumber) does not match actual release number ($AMOSReleaseNumber)"
        fi
}


####################################################################
#
# This unit collects General AMOS  version Information. 
# It collects the version number of the AMOS executables,libraries, 
# kernel drivers and version numbers of AM, LDAP and GSKit.
#
####################################################################
collectVersion() {
    print_to_stderr ".... Collecting Version Information ...."
    major_section "Version Information"

#   Prints version number of the AMOS libraries
    print ">>> AMOS Version Information:"
    /opt/pdos/bin/pdosversion

#   Executables in /opt/pdos/bin
    AMOS_EXECUTABLES="pdosauditd 
                      pdosaudview
                      pdosauthorize
                      pdosbkup
                      pdosbranchcfg
                      pdoscfg
                      pdoscollview 
                      pdosctl  
                      pdosd 
                      pdoshla
                      pdoslradm
                      pdoslrd
                      pdosnpd
                      pdospamcfg
                      pdosrgyimp 
                      pdosrstr
                      pdossudo  
                      pdosucfg 
                      pdosversion
                      pdoswdd  
                      pdoswhoami"

    HEADSIZE=2
    TAILSIZE=1

    for i in $AMOS_EXECUTABLES
    do
    /opt/pdos/bin/$i -V | $HEAD$HEADSIZE | $TAIL$TAILSIZE
    done
#   Other Executables
    /opt/pdos/sbin/policyview -V | $HEAD$HEADSIZE | $TAIL$TAILSIZE
#   Linux doesnot have se2pdos

    print ""

    if [ -f /opt/IBM/Tivoli/CommonAudit/client/lib/buildinfo.properties ] ; then
        print ""
        print ">>> Common Auditing and Reporting Service Version Information:"
	cat /opt/IBM/Tivoli/CommonAudit/client/lib/buildinfo.properties
    fi

    print ""
    print ">>> AM Version Information:"
    pdversion
    print "Build Number (pdadmin -V):"
    /opt/PolicyDirector/bin/pdadmin -V | $HEAD$HEADSIZE | $TAIL$TAILSIZE

    if [ -f /opt/IBM/Tivoli/SecUtilities/etc/version.incl ] ; then
        print ""
        print ">>> TivSecUtl Version Information:"
	grep TIVSEC_V /opt/IBM/Tivoli/SecUtilities/etc/version.incl
    fi

    print ""
    print ">>> LDAP Version Information"

#   There is no reliable way of finding the build number number of the LDAP 
#   client from the command line.
#   "ldapsearch -e" gives the major version number of LDAP and
#   It does not however say if a fixpack has been installed.
#   To find out the version of the patch installed on a machine, check for the existence of
#   the buildno.txt file and report the information in that file.
#   Basically try to gather as much information as is available. 


#   MAJOR version number
    print "Major Version Number (ldapsearch -e)."
    print "Note: If a LDAP fixpack is installed, the major version number does not change"
    if [ -x /opt/IBM/ldap/V6.0/bin/idsldapsearch ] ; then
       /opt/IBM/ldap/V6.0/bin/idsldapsearch -e
    elif [ -x /opt/ibm/ldap/V6.0/bin/idsldapsearch ] ; then
       /opt/ibm/ldap/V6.0/bin/idsldapsearch -e 
    elif [ -x /bin/idsldapsearch ] ; then
       /bin/idsldapsearch -e 
    elif [ -x /usr/bin/idsldapsearch ] ; then
       /usr/bin/idsldapsearch -e 
    elif [ -x /usr/ldap/bin/idsldapsearch ] ; then
       /usr/ldap/bin/idsldapsearch -e 
    elif [ -x /usr/IBMldap/bin/idsldapsearch ] ; then
       /usr/IBMldap/bin/idsldapsearch -e 
    else 
       print "Cannot find idsldapsearch utility"
    fi

#   PATCH build number
    case $PLATFORM in
       AIX) if [ -f /usr/ldap/web/buildno.txt ] ; then
               print ""
               print "Patch Build Number"
               cat /usr/ldap/web/buildno.txt
            fi 
            ;; 
       SunOS) if [ -f /opt/IBMldaps/web/buildno.txt ] ; then
                 print ""
                 print "Patch Build Number"
                 cat /opt/IBMldaps/web/buildno.txt
              fi
              ;;
       HPUX) if [ -f /usr/IBMldap/web/buildno.txt ] ; then
                print ""
                print "Patch Build Number"
                cat /usr/IBMldap/web/buildno.txt
             fi
             ;;
       Linux)
             if [ -f /usr/ldap/web/readme/buildno.txt ] ; then
                print ""
                print "Patch Build Number"
                cat /usr/ldap/web/readme/buildno.txt
             fi
             ;;
       *)  print "Unknown Platform"
    ;;
    esac

    print ""
    print ">>> GSKit Version Information:"

#   GSKit has a version command gsk7ver, but there are a couple of problems
#   with it:
#     1) The output is too verbose and there is no way to control the output
#     2) The command name changes with each major version
#   For gskit, we don't really care to know about the build number, just want to
#   find the version number of the GSKIT base package, so instead of using the
#   gskit version command, just query the installed packages on the machines 
#   to get the gskit version. List all the packages and then grep for gsk. 

    case $PLATFORM in
       AIX) lslpp -l | grep gsk 
            ;;
       SunOS) gskit_major=`pkginfo | grep gsk | awk '{print $2}'`
               print "$gskit_major: `pkginfo -l $gskit_major| grep VERSION`"
              ;;
       HPUX)
            swlist | grep gsk
            ;;
       Linux)
            rpm -qa | grep gsk
            ;;
       *)  print "Unknown Platform"
            ;;
     esac
}

####################################################################
#
# This unit describes what is collected for the General AMOS info
#
####################################################################
collectGeneral() {
    print_to_stderr ".... Collecting General AMOS Information ...."
    major_section "General AMOS Information"

    print ">>> AMOS bin and lib sums:"
    $SUM `find /opt/pdos/bin -type f`
    $SUM `find /opt/pdos/lib -type f`

#   Source Version Number: Includes the version number of all the files which
#   are linked into the executable. 
    print ""
    case $PLATFORM in
       AIX) 
	    print ">>> Source version info"
	    find /opt/pdos/bin -type f -exec $WHAT {} \;
	    print ""
	    find /opt/pdos/lib -type f -exec $WHAT {} \;
	    print ""
	    print ""
            ;;
       SunOS) 
	    print ">>> Source version info"
	    find /opt/pdos/bin -type f -exec $WHAT {} \; |fgrep -v SMI
	    print ""
	    find /opt/pdos/lib -type f -exec $WHAT {} \; | fgrep -v SMI
	    print ""
	    print ""
            ;;
       HPUX)
	    print ">>> Source version info"
	    find /opt/pdos/bin -type f -exec $WHAT {} \;
	    print ""
	    find /opt/pdos/lib -type f -exec $WHAT {} \;
	    print ""
	    print ""
            ;;
       Linux)
            ;;
       *)  print "Unknown Platform"
            ;;
     esac
   
    print ">>> AMOS configuration info:\n"
    $LS -l /var/pdos/etc

    print "\n\n>>> osseal.conf contents:"
    $CAT /var/pdos/etc/osseal.conf
    print "\n\n>>> pdosd.conf contents:"
    $CAT /var/pdos/etc/pdosd.conf
    print "\n\n>>> branch.conf contents:"
    $CAT /var/pdos/etc/branch.conf
    print "\n\n>>> pdosnpd.conf contents:"
    $CAT /var/pdos/etc/pdosnpd.conf
    print "\n\n>>> rpdb-current.xml contents:"
    $CAT /var/pdos/rpdb/rpdb-current.xml
    print "\n\n>>> /var/pdos dir contents (ls -lisR):"
    $LS -lsiR /var/pdos

    print "\n\n>>> Free space in major AMOS data directories:"
    print "/var/pdos/azn:"
    $DF /var/pdos/azn
    print "/var/pdos/audit:"
    $DF /var/pdos/audit
    print "/var/pdos/cred:"
    $DF /var/pdos/cred
    print "/var/pdos/log:"
    $DF /var/pdos/log
    print "/var/pdos/tracelogs:"
    $DF /var/pdos/tracelogs
    print "/var/pdos/rpdb:"
    $DF /var/pdos/rpdb

    print "\n\n>>> List of hla entries:"
    /opt/pdos/bin/pdoshla -l all
}

####################################################################
#
# This unit describes what is collected for the Logfile AMOS info
#
####################################################################
collectLogfile() {
    print_to_stderr ".... Collecting AMOS Logfile Information ...."
    
    print "\n\n>>> pdosd log file data":
    for i in /var/pdos/log/msg__pdosd.log*
    do
        print "+++ logfile: $i +++"
		  $TAIL$TAILSIZE $i
    done

    print "\n\n>>> pdosauditd log file data":
    for i in /var/pdos/log/msg__pdosauditd.log*
    do
        print "+++ logfile: $i +++"
		  $TAIL$TAILSIZE $i
    done

    print "\n\n>>> pdoswdd log file data":
    for i in /var/pdos/log/msg__pdoswdd.log*
    do
        print "+++ logfile: $i +++"
  		  $TAIL$TAILSIZE $i
    done

    print "\n\n>>> pdosnpd log file data":
    for i in /var/pdos/log/msg__pdosnpd.log*
    do
	print "+++ logfile: $i +++"
	          $TAIL$TAILSIZE $i
    done
    
    print "\n\n>>> PolicyUpdateDaemon log file data":
    for i in /var/pdos/log/msg__PolicyUpdateDaemon.log*
    do
	print "+++ logfile: $i +++"
	          $TAIL$TAILSIZE $i
    done
}

####################################################################
#
# This unit describes what is collected for the System info
#
####################################################################
collectSystem_AIX() {
 
    print "\n\n>>> Paging, VMM, etc:"
    /usr/sbin/lsps -a
 
    if [ -f /usr/bin/svmon ]; then
        print ""
        /usr/bin/svmon
        print "\nsvmon -S 0"
        /usr/bin/svmon -S 0
        print ""
    fi
 
    print "\n\n>>> Networking Information:"
    print "netstat -in ..."
    netstat -in
    print "\nnetstat -s ..."
    netstat -s

    print "\n\n>>> inittab info"
    /usr/sbin/lsitab -a

    print "\n\n>>> ipcs info"
    ipcs -a

    print "\n\n>>> OS Level Data:"
    /usr/bin/oslevel -q 2>&1
    /usr/bin/oslevel -g 2>&1
    /usr/bin/sum /unix
    /usr/bin/sum /usr/lib/boot/unix

    print "\n\n>>> S/W products:"
    /usr/bin/lslpp -ha

    print "\n\n>>> System dump info:"
    /usr/bin/sysdumpdev -l
    /usr/bin/sysdumpdev -L
}

collectSystem_SunOS() {
 
    print "\n\n>>> Networking Information:"
    print "netstat -in ..."
    netstat -in
    print "\nnetstat -s ..."
    netstat -s
    print "\nifconfig..."
    ifconfig -a

    print "\n\n>>> S/W products:"
    showrev -a
    print ""
    sysdef -D

    print "\n\n>>> ipcs info"
    ipcs -a

    print "\n>>> System (/etc/system) configuration parameters:"
    cat /etc/system | egrep -v  \^[*] | egrep [a-zA-Z]

    print "\n>>> /var/adm/messages data:"
    tail -200 /var/adm/messages

    print "\n>>> Loaded drivers:"
    modinfo
}
collectSystem_HPUX() {
     
    print "\n\n>>> Networking Information:"
    print "netstat -in ..."
    netstat -in
    print "\nnetstat -s ..."
    netstat -s

    print "\n\n>>> DLKM info:"
    /usr/sbin/kmadmin -S
    print ""
    /usr/bin/fgrep KM_INIT /etc/rc.config.d/kminit

    print "\n\n>>> ipcs info"
    ipcs -a

    print "\n\n>>> S/W products:"
    /usr/sbin/swlist

    print "\n\n>>> System dump info:"
    /sbin/crashconf -v
}
collectSystem_Linux() {
     
    for pp in `/bin/ps -ef|grep -E "pdos.*d|KPCF|NCT"|grep -v grep|awk '{print $2}'`
      do
        print "\n\n"
        print "/proc/" $pp
        print "oooooooooooo"
        ls -l /proc/$pp/* 2>/dev/null
        print "\ncmdline: " `cat /proc/$pp/cmdline` "\n"
        print "\nenviron: " `cat /proc/$pp/environ` "\n"
        print "\nstat"
        cat /proc/$pp/stat
        print "\nstatm"
        cat /proc/$pp/statm
        print "\nstatus"
        cat /proc/$pp/status
      done

    print "\n\n>>> Paging, Memory, etc:"
    for aa in swaps meminfo cmdline locks modules slabinfo stat version
    do
        print "\n\n>>> /proc/$aa"
        cat /proc/$aa
    done

    print "\n\n>>> Open File Information"
    if [ -x /usr/bin/lsof ]; then
        /usr/bin/lsof
    fi

    if [ -x /usr/sbin/lsof ]; then
        /usr/sbin/lsof
    fi

    print "\n\n>>> ipcs info"
    ipcs -a

    print "\n\n>>> OS Level Data:"
    cat /etc/*-release
    print ""
    cat /proc/version
    print "\n>>> /boot contents:"
    ls -alt /boot/*

#    This gives no clear info.
#    print "\n\n>>> AMOS and related packages:"
#    for pa in PDRTE PDAuthADK PDOSrte-PDOSruntime PDOSmsg-en_US
#    do
#        print "\n>>> $pa:"
#        /bin/rpm -V $pa
#    done

    print "\n\n>>> S/W products:"
    /bin/rpm -qa | sort
}
####################################################################
collectSystem() {
    print_to_stderr ".... Collecting General System Information ...."

    major_section "General System Information"
    
    print "\n>>> uname info:"
    uname -a
    print ""

    print ">>> Current running processes ($PS):"
    $PS
 
    print "\n\n>>> File System Information"
		  $MOUNT

    print ""
		  $DF

    if [ -f /usr/bin/vmstat ]; then
        print "\n\n>>> Live vmstat samples"
        /usr/bin/vmstat 2 5
    fi
    
    if [ -f /usr/bin/iostat ]; then
        print "\n\n>>> Live iostat samples"
        /usr/bin/iostat 2 5
    fi

		  # This is now very platform specific
		  collectSystem_$PLATFORM

}

####################################################################

 #####  #######    #    ######  #######
#     #    #      # #   #     #    #
#          #     #   #  #     #    #
 #####     #    #     # ######     #
      #    #    ####### #   #      #
#     #    #    #     # #    #     #
 #####     #    #     # #     #    #

#     #    #      ###   #     #
##   ##   # #      #    ##    #
# # # #  #   #     #    # #   #
#  #  # #     #    #    #  #  #
#     # #######    #    #   # #
#     # #     #    #    #    ##
#     # #     #   ###   #     #

####################################################################
####################################################################
####################################################################

PROGRAM=`basename $0`
SetMachineType
ProcessArgs $@

buildHeader

if [ "$Version" = "yes" ] ; then
   collectVersion
fi
if [ "$VerifyRelease" = "yes" ] ; then
    verifyReleaseLevel $ReleaseNumber
fi
if [ "$General" = "yes" ] ; then
   collectVersion
   collectGeneral
fi
if [ "$Logfile" = "yes" ] ; then
   collectLogfile
fi
if [ "$System" = "yes" ] ; then
   collectSystem
fi

print_to_stderr "Done..."
print "\n\n>>> End of Collected Data"

exit 0




